/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * private.h
 * Private definitions, data structures, macros and functions for libpathport
 * Copyright (C) 2006 Simon Newton
 */

#ifndef _defined_pathport_private_h
#define _defined_pathport_private_h

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <pathport.h>
#include <pathport/packets.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

static const uint16_t PATHPORT_PROTO = 0xED01;    // protocol id
static const uint16_t PATHPORT_VERSION = 0x0200;  // current version
static const uint32_t PATHPORT_PORT = 3792;      // udp port

/* pathport ids */
typedef enum {
  PATHPORT_ID_BROADCAST = 0xffffffff,
  PATHPORT_ID_MCAST_ALL = 0xefffedff,
  PATHPORT_ID_MCAST_DATA = 0xefffed01,
  PATHPORT_ID_MCAST_MANAGE = 0xefffed02
} pathport_id;

static const uint32_t PATHPORT_ADDR_DATA = PATHPORT_ID_MCAST_DATA;    // data multicast address
static const uint32_t PATHPORT_ADDR_CONFIG = PATHPORT_ID_MCAST_MANAGE;  // config multicast address
static const uint32_t PATHPORT_ADDR_STATUS =PATHPORT_ID_MCAST_ALL;    // status multicast address

enum { PATHPORT_MAC_LENGTH = 6 };
enum { PATHPORT_NAME_LENGTH = 64 };

static const uint16_t PATHPORT_XDMX_DATA_FLAT = 0x0101;    // opcode for dmx
static const uint16_t PATHPORT_XDMX_DATA_RELEASE = 0x0103;  // opcode for dmx release

static const int PATHPORT_DMX_LENGTH = 512;

// non pathport specific
static const int RECV_NO_DATA = 1;

static const int TRUE =1;
static const int FALSE =0;

#define SA struct sockaddr
#define SI struct in_addr

#define min(a,b)    ((a) < (b) ? (a) : (b))
#define max(a,b)    ((a) > (b) ? (a) : (b))

// byte ordering macros
#define bswap_16(x)  ((((x) >> 8) & 0xff) | (((x) & 0xff) << 8))

#define short_gethi(x) ((0xff00 & x) >> 8)
#define short_getlo(x)  (0x00ff & x)

#define pathport_error_malloc() pathport_error("%s : malloc error %s" , __FUNCTION__, strerror(errno) )
#define pathport_error_realloc() pathport_error("%s : realloc error %s" , __FUNCTION__, strerror(errno) )
#define pathport_error_nullnode() pathport_error("%s : argument 1 (pathport_node) was null" , __FUNCTION__ )

// check if the node is null and return an error
#define return_if_null(node) if (node == NULL) { \
  pathport_error_nullnode(); \
  return PATHPORT_EARG; \
}

// return if the node is off
#define return_if_off(node) if (node->state.mode == PATHPORT_OFF) { \
  pathport_error("%s : node is not on" , __FUNCTION__ ); \
  return PATHPORT_ESTATE; \
}

// return if the node is on
#define return_if_on(node) if (node->state.mode == PATHPORT_ON) { \
  pathport_error("%s : node is not on" , __FUNCTION__ ); \
  return PATHPORT_ESTATE; \
}


// packet callbacks have a function and some user data
typedef struct {
  int (*fh)(pathport_node n, void *p, void *data);
  void *data;
} callback_t;


// The dmx callback is triggered when a dmx packet arrives
typedef struct {
  int (*fh)(pathport_node n, unsigned int universe, unsigned int length, const uint8_t *data, void *d);
  void *data;
} dmx_callback_t;


// the status of the node
typedef enum {
  PATHPORT_OFF,
  PATHPORT_ON
} node_status_t;


// struct to hold the state of the node
typedef struct {
  SI ip_addr;
  SI data_addr;
  SI config_addr;
  SI status_addr;
  uint32_t pathport_id;    // pathport id
  int verbose;
  uint8_t hw_addr[PATHPORT_MAC_LENGTH];
  uint16_t packet_count;
  pathport_manuf manuf;
  pathport_class cls;
  pathport_class_type type;
  node_status_t mode;
  char name[PATHPORT_NAME_LENGTH + 1];    // null terminated
} node_state_t;


/**
 * The main node structure
 */
typedef struct pathport_node_s{
  int sd[PATHPORT_MAX_SD];              // the sockets
  node_state_t state;                  // the state struct
  dmx_callback_t dmx_callback;            // dmx callback
  callback_t callbacks[PATHPORT_LAST_HANDLER];    // callbacks
  uint8_t *dmx[PATHPORT_MAX_UNIVERSES];
} pathport_node_t;


typedef pathport_node_t *node;


// Global variables
extern char *pathport_errstr;


// Function definitions follow
// pathport.c
void pathport_error(const char *fmt, ...);

//transmit.c
int pathport_tx_send_arprep(node n, pathport_id dst);
int pathport_tx_send_dmx(node n, uint8_t universe, int length, const uint8_t *data);
int pathport_tx_send_dmx_release(node n, int uni);
int pathport_tx_send_getreply(node n, pathport_id dst, pathport_param *params, unsigned int num);

//receive.c
int pathport_rx_handle(node n, pathport_ipacket p);

// network.c
ssize_t pathport_net_recv(node n, pathport_ipacket p, int block);
int pathport_net_send(node n, pathport_ipacket p);
int pathport_net_init(node n, const char *ip);
int pathport_net_start(node n);
int pathport_net_close(node n);

#endif
